@@ -212,6 +212,13 @@ class ObjectArray(awkward.array.base.AwkwardArrayWithContent):
     def _hasjagged(self):
         return False
 
+    @property
+    def counts(self):
+        raise TypeError("{0} has no 'counts' array".format(type(self).__name__))
+
+    def regular(self):
+        return self.numpy.array(self)
+
     def _reduce(self, ufunc, identity, dtype, regularaxis):
         raise TypeError("cannot call reducer on object array")
 
@@ -224,6 +231,17 @@ class ObjectArray(awkward.array.base.AwkwardArrayWithContent):
         out._content = arrays[0]._content.__class__.concatenate([a._content for a in arrays])
         return out
 
+    def _util_pandas(self, seen):
+        import awkward.pandas
+        if id(self) in seen:
+            return seen[id(self)]
+        else:
+            out = seen[id(self)] = self.copy()
+            out.__class__ = awkward.pandas.mixin("ObjectSeries", self)
+            if isinstance(self._content, awkward.array.base.AwkwardArray):
+                out._content = out._content._util_pandas(seen)
+            return out
+
 ####################################################################### strings
 
 class StringMethods(object):
@@ -556,6 +574,10 @@ class StringArray(StringMethods, ObjectArray):
             out = self._content[where]
             return self.__class__(out.starts, out.stops, out.content, self.encoding)
 
+    def regular(self):
+        self._valid()
+        return self.numpy.array(self)
+
     @property
     def iscompact(self):
         return self._content.iscompact
@@ -584,3 +606,14 @@ class StringArray(StringMethods, ObjectArray):
 
     def fillna(self, value):
         return self
+
+    def _util_pandas(self, seen):
+        import awkward.pandas
+        if id(self) in seen:
+            return seen[id(self)]
+        else:
+            out = seen[id(self)] = self.copy()
+            out.__class__ = awkward.pandas.mixin("StringSeries", self)
+            if isinstance(self._content, awkward.array.base.AwkwardArray):
+                out._content = out._content._util_pandas(seen)
+            return out
